#!/usr/bin/env python3
import json
import logging
import os
import platform
import re
import subprocess
import sys
from argparse import ArgumentParser
from datetime import datetime
from os import path, system
from subprocess import getstatusoutput, getoutput
from sys import exit
from time import sleep

# 创建一个自定义的日志记录器
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

# 创建一个自定义的日志处理器，设置其输出格式
formatter = logging.Formatter('%(asctime)s [%(levelname)s] %(filename)s:%(lineno)d | %(funcName)s | %(message)s')
handler = logging.StreamHandler(sys.stdout)
handler.setFormatter(formatter)
logger.addHandler(handler)

# 业务列表
bl_str = 'EDSNode  EDSNodeH  JSNode  JSNodeH  KANode  KSNode  KSNodeA  KSNodeH  LSNode  QYNode'.lower()
business_lists = str(bl_str).replace("  ", "\n").split("\n")
business_dict = {}
s = 1
for b in business_lists:
	business_dict[s] = b
	s += 1

arch = platform.machine().lower()
if str(arch) == "x86_64".lower():
	arch = "amd64"
if str(arch) == "aarch64".lower():
	arch = "arm64"
if str(arch) == "aarch".lower() or str(arch) == "aarch32".lower():
	arch = "arm32"
if str(arch) == 'armv7l'.lower():
	arch = "arm32"
arch_list = ["arm64", "amd64", "arm32"]
if arch not in arch_list:
	logger.error(f"暂不支持当前系统该架构: {arch}")
	exit(1)


def check_service_listen():
	"""
	检测服务端口是否启动
	:return:
	"""
	s = 0
	while True:
		print(f"[ {datetime.now()} ]- 正在检查监听端口,每次间隔5秒,超时 120 秒,倒计时 [ {120 - s}]秒", end="\r")
		if system("lsof -i:1024") == 0:
			return True
		sleep(5)
		s += 5
		if s > 120:
			logger.error("监听端口超时")
			return False


def set_timezone(timezone="Asia/Shanghai"):
	"""
	设置系统时区
	:param timezone: 时区字符串
	"""
	cmd = f"timedatectl set-timezone {timezone}"
	if system(cmd) != 0:
		# 改进日志记录，提供更多错误信息
		logger.error(f"无法设置时区为{timezone}：{cmd}")
		# 考虑到异常处理，不再需要手动exit
		# 可以抛出该异常，由上层调用者决定如何处理
		raise
	else:
		logger.info(f"成功设置时区为{timezone}")


def check_process(file_):
	"""
	检查进程是否启动
	:param file_: 进程文件名
	:return:
	"""
	while True:
		print(f"[ {datetime.now()} ]正在检查进程启动,每次间隔5秒...", end="\r")
		cmd = f"ps -ef| grep {file_} | grep -v grep"
		if getstatusoutput(cmd)[0] == 0:
			logger.info("进程已启动")
			return True
		sleep(5)
		return False


def show():
	logger.info("服务启动成功,请通过下面的地址访问后台")
	ip_str = getoutput("hostname -I")
	ips = str(ip_str).replace("127.0.0.1", '').split(" ")
	print("\n")
	for ip in ips:
		if '.' in str(ip):
			print(f"http://{ip}:1024")
		else:
			if len(ip) >= 8:
				print(f"http://[{ip}]:1024")
	print("\n")


def service_show():
	"""
	查看服务状态
	:return:
	"""
	print("可使用下面的命令进行管理")
	print("systemctl enable com.liumou.ttnode.service # 设置开机自启(已设置)")
	print("systemctl disable com.liumou.ttnode.service # 取消开机自启")
	print("systemctl restart com.liumou.ttnode.service # 重启服务")
	print("systemctl start com.liumou.ttnode.service # 启动服务")
	print("systemctl status com.liumou.ttnode.service # 查看服务状态")
	print("journalctl  -u com.liumou.ttnode.service # 查看日志")


def service_text(run, work, stop, cache, bs: str):
	"""
	服务脚本生成
	:param bs:
	:param cache: 缓存目录
	:param stop: 停止脚本
	:param run: run.sh文件路径
	:param work: 工作目录
	:return:
	"""
	if not args_service:
		logger.debug("略过生成服务脚本")
		return
	# tp = "-t"
	# if str(bs).lower() == "bynode".lower():
	# 	tp = "-force_node"
	# 生成并执行一个启动脚本，用于启动指定的进程并保持运行
	# 参数: file - 需要启动的进程对应的文件名
	# 返回值: 无
	sd = "/etc/systemd/system/"
	for p in [sd, run, work]:
		if not path.exists(p):
			logger.error(f"路径 {p} 不存在")
			exit(3)
	file = path.join(sd, "com.liumou.ttnode.service")
	service = "com.liumou.ttnode"
	ExecStart = f"{run} -c {cache} -t {bs}"
	if not t_:
		ExecStart = f"{run} -c {cache}"
	txt = f"""[Unit]
Description=This is a TTNODE
After=network.target
StartLimitIntervalSec=5

[Service]
Type=simple
User=root
ExecStart={ExecStart}
# 在停止服务之前执行的命令
ExecStop={stop}
PIDFile=/run/ttmanager/ttrun.pid
#WorkingDirectory={work}
StandardOutput=journal
# 终止该服务所有进程
KillMode=control-group
Restart=on-failure
RestartPreventExitStatus=255
StandardError=inherit
SyslogIdentifier={service}

[Install]
WantedBy=multi-user.target
Alias={service}.service
"""
	try:
		with open(file=file, mode='w', encoding='utf8') as w:
			w.write(txt)
	except Exception as e:
		logger.error(e)
		exit(2)
	if getstatusoutput("systemctl -all | grep com.liumou.ttnode.service")[0] == 0:
		getoutput("systemctl disable com.liumou.ttnode.service")
	r = getstatusoutput("systemctl daemon-reload")
	if r[0] != 0:
		logger.error("systemctl daemon-reload 执行失败")
		print(r[1])
		exit(3)
	c = "systemctl enable com.liumou.ttnode.service"
	r = getstatusoutput(c)
	if r[0] != 0:
		logger.warning("systemctl enable com.liumou.ttnode.service 执行失败")
	if system("systemctl start com.liumou.ttnode.service") != 0:
		logger.error("systemctl start com.liumou.ttnode.service 执行失败")
		exit(5)
	else:
		logger.info("服务启动成功,进入下一步检测端口监听...")


def stop_service(file: str):
	"""
	停止服务脚本生成
	:param file:
	:return:
	"""
	# 生成并执行一个终止脚本，用于关闭指定的进程和重启服务
	# 参数: file - 需要关闭的进程对应的文件名
	# 返回值: 无
	res = getstatusoutput(f"pgrep {file}")
	if res[0] == 0:
		kill = getstatusoutput(f"kill -9 {res[1]} ")
		if kill[0] != 0:
			logger.error(f"{file} 进程关闭失败")
			exit(6)


class InstallTtNode:
	def __init__(self, u: str, tt_dir, cache, index: str):
		"""
		初始化安装TTNode的类实例。

		:param u: 下载执行文件的URL。
		:param tt_dir: 二进制可执行文件存放路径。
		:param cache: 数据缓存路径。
		"""
		self.index = index
		self.cache_dir = cache  # 数据缓存路径
		self.url = u  # 执行文件URL
		self.url_shell_stop = ""  # 停止脚本URL
		self.file = str(self.url).split('/')[-1]  # 提取URL中的文件名
		self.tt_bin_file = ""  # 二进制执行文件绝对路径
		self.tt_bin_dirs = tt_dir  # 二进制文件根目录
		self.run_shell = path.join(self.tt_bin_dirs, "run.sh")  # 运行脚本路径
		self.shell_stop = path.join(self.tt_bin_dirs, 'stop.sh')
		self.package_manager = "apt"

	def init(self):
		"""
		初始化安装环境，检查必备条件。

		此函数不接受参数，也不返回任何值。
		主要执行以下任务：
		1. 设置系统时区为Asia/Shanghai。
		2. 检查缓存目录是否存在，如果不存在且业务类型为ByNode，根据var_p的值决定是直接退出还是尝试创建缓存目录。
		3. 检查可执行文件目录是否存在，如果不存在则尝试创建。
		4. 停止之前运行的服务。
		5. 打印配置信息，包括执行文件下载链接、保存路径、缓存路径和文件路径。
		"""
		# 检查缓存目录
		if not path.isdir(self.cache_dir) and business.lower() == "ByNode".lower():
			if not var_p:
				logger.error(f"缓存目录不存在,请先创建或者部署bynode业务 -> {self.cache_dir}")
				exit(1)
			else:
				r = getstatusoutput(f"mkdir -p {self.cache_dir}")
				if r[0] != 0:
					logger.error(f"无法创建缓存文件目录: {self.cache_dir}")
					print(r[1])
					exit(2)

		# 检查并创建可执行文件目录
		if not path.isdir(self.tt_bin_dirs):
			r = getstatusoutput(f"mkdir -p {self.tt_bin_dirs}")
			if r[0] != 0:
				logger.error(f"无法创建可执行文件目录: {self.tt_bin_dirs}")
				print(r[1])
				exit(2)

		# 设置二进制执行文件绝对路径
		self.tt_bin_file = path.join(self.tt_bin_dirs, self.file)
		self.url_shell_stop = f"{self.index}///stop.sh".replace("////", "/").replace("///", "/")

		# 停止服务
		stop_service(self.run_shell)
		stop_service(self.tt_bin_file)

		# 打印配置信息
		print("----------------配置信息------------")
		print(f"------执行文件下载链接: {self.url} --")
		print(f"------终止脚本下载链接: {self.url_shell_stop} --")
		print(f"------执行文件保存路径: {self.tt_bin_dirs} --")
		print(f"------缓存路径: {self.cache_dir} --")
		print(f"------二进制文件保存路径: {self.tt_bin_file} --")
		print(f"------终止脚本保存路径: {self.shell_stop} --")

	def download(self):
		"""
		下载执行文件和相关脚本。

		该方法不接受参数，也不直接返回任何值，但会在下载成功或失败时通过日志记录相关信息。

		主要步骤包括：
		1. 删除已存在的执行文件。
		2. 通过wget命令下载指定URL的文件。
		3. 根据下载结果记录日志信息。
		"""
		logger.info("开始下载执行文件...")
		# 删除已存在的执行文件
		system(f"rm -f {self.tt_bin_file}")

		# 下载执行文件，并根据下载结果记录日志
		c = f"wget -q {self.url} -O {self.tt_bin_file}"
		r = getstatusoutput(c)
		if r[0] != 0:
			logger.error(f"二进制执行文件下载失败: {self.tt_bin_file}")
			print(r[1])
			print(c)
			exit(2)  # 下载失败时终止程序
		else:
			logger.info(f"二进制执行文件下载成功: {self.tt_bin_file}")
		system(f"rm -f {self.shell_stop}")
		c = f"wget -q {self.url_shell_stop} -O {self.shell_stop}"
		r = getstatusoutput(c)
		if r[0] != 0:
			logger.error(f"终止脚本下载失败: {self.shell_stop}")
			print(r[1])
			print(c)
			exit(2)  # 下载失败时终止程序
		else:
			logger.info(f"终止脚本下载成功: {self.shell_stop}")

	def chmod(self):
		"""
		修改脚本文件的执行权限。
		"""
		# 修改执行文件和定时任务文件的权限
		c = f"chmod +x {self.tt_bin_file} {self.shell_stop}"
		r = getstatusoutput(c)
		if r[0] != 0:
			logger.error(f"修改权限失败: {self.tt_bin_file}")
			print(r[1])
			exit(3)

	def run(self):
		"""
		运行TTNode执行文件。

		此方法不接受参数，并且没有返回值。
		主要步骤包括切换到执行文件所在目录，生成并运行一个保活脚本。
		如果保活脚本无法生成，则记录错误信息并退出程序。
		"""
		os.chdir(self.tt_bin_dirs)  # 切换到执行文件目录

		# 生成保活脚本的命令行指令，并运行该指令
		cre = f"./{self.file} -c {self.cache_dir} -t {business} -g > /dev/null"
		logger.debug(f"生成保活脚本: {cre}")
		r = getstatusoutput(cre)
		if r[0] != 0:
			logger.error("无法生成保活脚本...")
			print(r[1])
			exit(4)  # 遇到错误，无法生成保活脚本时，程序退出

	def get_packages(self):
		"""

		:return:
		"""
		if system("which yum") == 0:
			self.package_manager = "yum"
			if update_:
				system(f"{self.package_manager} clean all;{self.package_manager} makecache")
			return
		if system("which opkg") == 0:
			self.package_manager = "opkg"
		if update_:
			system(f'{self.package_manager} update > /dev/null')
		logger.debug(f"使用默认管理器: {self.package_manager}")

	def bynode(self):
		"""
		部署ByNode业务相关的组件和配置。

		该函数检查当前业务是否为ByNode业务，如果是，则根据系统类型选择合适的包管理工具（apt或yum），
		更新包列表，安装Docker，以及安装ByNode所需的组件（如jq和iproute2）。

		参数:
		self - 对象的自引用，允许访问类的属性和方法。

		返回值:
		无返回值。
		"""
		# 检查当前业务是否为ByNode业务
		if str(business).lower() != "bynode".lower():
			return
		self.get_packages()

		# 安装Docker
		if system("which docker > /dev/null") != 0:
			# 使用curl通过阿里云镜像安装Docker
			if system(f"curl -fsSL https://get.docker.com | bash -s docker --mirror Aliyun") != 0:
				logger.error("Docker安装失败,请先手动安装再运行脚本或者使用其他业务")
				return
			else:
				logger.info("Docker安装成功")

		# 安装jq
		if system("which jq > /dev/null") != 0:
			if system(f"{self.package_manager} install -y -qq jq") != 0:
				logger.error("[ jq ]安装失败,请先手动安装再运行脚本或者使用其他业务")
				return
		# 安装iproute2
		if system("which ss > /dev/null") != 0:
			if system(f"{self.package_manager} install -y -qq iproute2") != 0:
				logger.error("[ iproute2 ]安装失败,请先手动安装再运行脚本或者使用其他业务")
				return

	def ConfigDocker(self):
		"""
		配置Docker驱动。
		此函数检查并修改Docker的配置文件，以设置指定的执行选项和镜像仓库。如果系统当前使用的驱动不是cgroupfs，将进行配置更改。
		配置过程中会备份原有的配置文件，并在完成配置后重载Docker配置并重启Docker服务。
		"""
		if business.lower() != "bynode".lower():
			logger.debug("当前业务非ByNode,取消Docker配置")
			return
		logger.info("正在进入docker配置...")
		if not docker_:
			logger.warning("未添加-docker参数,取消Docker配置")
			return
		if system("which docker") != 0:
			logger.error("Docker未安装,请先安装再运行脚本或者使用其他业务")
			return
		file = '/etc/docker/daemon.json'  # Docker配置文件路径
		# 确保Docker配置目录存在
		if not path.isdir('/etc/docker'):
			if system('mkdir -p /etc/docker') != 0:
				logger.error("无法创建Docker配置目录")
				exit(5)

		bak = f"{file}.bak_liumou"  # 配置文件备份路径

		# 配置驱动
		logger.debug("正在进行docker配置...")
		new_config = {
			"exec-opts": ["native.cgroupdriver=cgroupfs"],
			"registry-mirrors": ["https://docker.mirrors.ustc.edu.cn"]
		}
		txt = json.dumps(new_config, indent=4)  # 新的Docker配置内容
		if not os.path.isfile(bak):
			cmd = f"cp -rf {file} {bak}"
			system(cmd)

		try:
			with open(file=file, mode='w', encoding="utf-8") as f:
				f.write(txt)  # 写入新的配置
		except Exception as e:
			logger.error('无法写入Docker配置', e)
			exit(6)
		logger.debug("Docker配置成功,正在重载配置并重启Docker...")
		if system('systemctl daemon-reload') != 0:
			logger.error("系统配置重载失败")
			exit(1)
		if system('systemctl restart docker.service docker.socket') != 0:
			logger.error("Docker重启失败,请检查配置")
			exit(2)

		# 检查驱动是否修改成功
		if subprocess.check_output(['docker', 'info']).decode().find('cgroupfs') == -1:
			logger.error("Docker驱动修改失败")
			exit(3)

	def install_vnstat(self):
		if system("which vnstat > /dev/null") != 0:
			if system(f"{self.package_manager} install -y vnstat") != 0:
				logger.error("vnstat安装失败")
				return

	def start(self):
		"""
		开始安装和配置过程。
		"""
		if args_set_time:
			set_timezone()
		self.init()  # 初始化环境
		self.install_vnstat()  # 安装vnstat
		self.bynode()  # 部署ByNode业务
		self.ConfigDocker()
		self.download()  # 下载文件
		self.chmod()  # 修改权限
		self.run()  # 运行
		service_text(run=self.tt_bin_file, work=self.tt_bin_dirs, stop=self.shell_stop, cache=self.cache_dir,
		             bs=business)
		if check_process(self.tt_bin_file):
			if check_service_listen():
				logger.info("端口监听成功")
			else:
				logger.error("端口监听失败")
				exit(5)
		service_show()
		show()
		logger.info("Configuration completed")
		print("打开APP，进入心愿界面，点击右边中间的闪电输入我的邀请码支持一下吧")
		print(512267)
		print("电信光猫超级密码查询请联系")
		print("sytx20170401")


def is_valid_version(version):
	"""
	检查版本号格式是否有效
	"""
	return re.match(r'^1\.\d+\.\d+$', version) is not None


def is_valid_url(url):
	"""
	检查URL是否有效。这里使用了简单的正则表达式，实际项目中可能需要更复杂的验证。
	"""
	return re.match(r'^https?://', url) is not None


def check_os():
	"""
	检查系统是否支持
	:return:
	"""
	if platform.system().lower() != 'Linux'.lower():
		logger.error("仅支持Linux系统")
		exit(2)
	res = getoutput("cat /etc/os-release  | grep ^ID=").split("=")[1].lower().strip()
	# 去除单引号和双引号
	res = res.replace("'", "").replace('"', '')
	os_list = ["openwrt", "ubuntu", "debian", "centos", "armbian"]
	logger.info("当前支持的系统:")
	print(os_list)
	logger.info(f"当前系统为: {res}")
	if res not in os_list:
		logger.error("不支持此操作系统")
		exit(2)


def select_business():
	"""
	选择业务
	:return:
	"""
	global business
	if business is None:
		while True:
			print("请选择业务:")
			for index in business_dict:
				print(f"{index} : {business_dict[index]}")
			use = input("请输入序号: ")
			try:
				use = int(use)
			except Exception as e:
				logger.error(f"错误: {e}")
				continue
			if 0 < int(use) <= len(business_dict):
				business = business_dict[int(use)]
				break


if __name__ == '__main__':
	business = 'qynode'
	arg = ArgumentParser(description='当前脚本版本: 3.6', prog="甜糖二进制安装")
	h = '设置执行文件存放目录,默认: /data/tt/bin(不存在则自动创建)'
	arg.add_argument('-d', '--dir', type=str, help=h, default="/data/tt/bin", required=False)
	h = '设置数据缓存目录,默认：/data/tt/node(不存在则终止,[ByNode]业务除外)'
	arg.add_argument('-c', '--cache', type=str, help=h, default="/data/tt/node", required=False)

	h = '自定义二进制文件下载链接,当设置此参数后后面的版本相关设置将无效'
	arg.add_argument('-u', '--url', type=str, help=h, required=False)
	index_default = 'https://down.liumou.site/pve/tt'
	h = f"设置脚本文件下载索引主目录,默认: {index_default}"
	arg.add_argument('-i', '--index', type=str, help=h, required=False, default=index_default)

	h = f'设置业务类型,默认：{business}'
	arg.add_argument('-b', '--business', type=str, help=h, required=False)
	h = '设置部署版本,默认: 1.5.18'
	arg.add_argument('-v', '--ver', type=str, help=h, default="1.5.18", required=False)
	arg.add_argument('-l', action='store_true', help='查看业务列表')
	arg.add_argument('-f', action='store_true', help='清除已有的系统定时任务后再添加任务,默认不清除')
	arg.add_argument('-p', action='store_true', help='自动创建缓存目录')
	arg.add_argument('-t', action='store_false', help='取消指定服务启动业务类型')
	arg.add_argument('-st', action='store_false', help='取消时区设置')
	arg.add_argument('-service', action='store_false', help='取消Service创建')
	arg.add_argument('-update', action='store_true', help='源索引(apt update)更新')
	arg.add_argument('-docker', action='store_true', help='强制进行docker安装和配置,不管之前有没有安装')
	args = arg.parse_args()
	dir_ = args.dir
	args_cache = args.cache
	url_arg = args.url
	business = args.business
	l_ = args.l
	f_ = args.f
	t_ = args.t
	args_set_time = args.st
	args_service = args.service
	var_p = args.p
	update_ = args.update
	docker_ = args.docker
	ver_args = args.ver
	args_index = args.index
	check_os()
	select_business()
	if not is_valid_version(args.ver):
		arg.error("提供的版本号格式不正确。应为X.X.X格式。")
	if url_arg:
		url_ = url_arg
	else:
		if arch == 'arm32':
			logger.warning("ARM32仅更新到1.5.12版本")
			ver_args = '1.5.12'
		url_ = f'https://down.liumou.site/ttmanager2/{ver_args}/ttmanager_{arch}'
	if l_:
		for i in business_lists:
			print(str(i))
		exit(0)
	it = InstallTtNode(tt_dir=dir_, cache=args_cache, u=url_, index=args_index)
	it.start()
